from typing import Literal

PLAYER_ACTION_ACTION = Literal['SetHotbar', 'SetReducedDebug', 'CloseInv', 'GiveItems', 'NoKeepInv', 'SetHandCrafting', 'BossBar', 'SetBossBar', 'ParticleSphere', 'SetVelocity', 'Particle', 'AddInvRow', 'NoNatRegen', 'DisplayLightning', 'Damage', 'SendAnimation', 'SetXPProg', 'SetInventory', 'TpSequence', 'Heal', 'SetSpawnPoint', 'SetInventoryKept', 'LaunchUp', 'GetTargetEntity', 'ForceFlight', 'LoadInv', 'ChatStyle', 'ChatColor', 'Kick', 'ProjColl', 'MiscAttribute', 'SpectateTarget', 'HurtAnimation', 'SurvivalMode', 'GmSurvival', 'DisplayBellRing', 'SetStatus', 'SetCursorItem', 'SetAbsorption', 'SetFireTicks', 'CombatAttribute', 'SetGamemode', 'RemoveInvRow', 'WakeUpAnimation', 'DisableBlocks', 'SetScoreObj', 'L SetHealth', 'ParticleEffect', 'ClearInv', 'SetFreezeTicks', 'SetGliding', 'SetRotation', 'ClearItems', 'OpenSign', 'SetFlying', 'DisplayBlockOpen', 'SetHandItem', 'SendAdvancement', 'ClearChat', 'SetMenuItem', 'LaunchToward', 'SetArmor', 'DisplayGateway', 'GiveSaturation', 'DisplayEquipment', 'GiveExp', 'FaceLocation', 'ClearScoreboard', 'ActionBar', 'SetChatTag', 'ShiftWorldBorder', 'DisplaySignText', 'SetSpeed', 'ExpandInv', 'LaunchProj', 'NoProjColl', 'ShowDisguise', 'ParticleCuboidA', 'PlaySound', 'SetCompass', 'MobDisguise', 'EnableBlocks', 'OpenBlockInv', 'ParticleCircleA', 'RemoveBossBar', 'ClearBars', 'SetEquipment', 'GiveRngItem', 'SetDropsEnabled', 'SendToPlot', 'RemovePotion', 'RemoveEffect', 'DisplayFracture', 'SetEntityHidden', 'SetSidebar', 'AllowDrops', 'Vibration', 'SetSlot', 'ParticleRay', 'ParticleCuboid', 'SendMessageSeq', 'SendDialogue', 'SetNamePrefix', 'ClearDispBlock', 'SetRainLevel', 'Undisguise', 'ParticleSpiralA', 'InstantRespawn', 'SetScore', 'SetNameColor', 'ReachAttribute', 'SetAtkSpeed', 'DisablePvp', 'SetTickRate', 'PlayEntitySound', 'ReplaceProj', 'SetExp', 'MiningAttribute', 'KBAttribute', 'MovementAttribute', 'ParticleSpiral', 'FallingAttribute', 'SetAllowFlight', 'SetMaxHealth', ' RemoveBossBar ', 'SetFogDistance', 'AdventureMode', 'GmAdventure', 'SpectatorMode', 'DispHeadTexture', 'ClearPotions', 'ClearEffects', 'SetTabListInfo', 'EnablePvp', 'PromptPurchase', 'HideDisguise', 'ScoreLineFormat', ' SetBossBar ', 'SetSkin', 'SpectatorCollision', 'SetNameVisible', 'SetInvulTicks', 'EnableFlight', 'SetStingsStuck', 'RemoveScore', 'DisallowDrops', 'SetExhaustion', 'ParticleCircle', 'DisplayBlock', 'RideEntity', 'WeatherRain', 'RmWorldBorder', 'ResourcePack', ' SetInvName ', 'GiveExhaustion', 'Teleport', 'SetAllowPVP', 'DisableFlight', 'SetVisualFire', 'SetDisguiseVisible', 'SetArrowsStuck', 'GetItemCooldown', 'SetItems', 'KeepInv', 'ReplaceItems', 'ReplaceItem', 'SendMessage', 'SetSlotItem', 'PlaySoundSeq', 'ParticleLineA', 'Respawn', 'SetInvName', 'SetItemCooldown', 'SetPlayerWeather', 'SendHover', 'SetShoulder', 'SetAirTicks', 'DisplayPickup', 'SetWorldBorder', 'SetPlayerTime', 'GiveFood', 'NatRegen', 'GivePotion', 'GiveEffect', 'RemoveItems', 'BoostElytra', 'SaveInv', 'OpenBook', 'SetHealth', 'BlockDisguise', 'RollbackBlocks', 'NoDeathDrops', 'WalkSpeed', 'SetCollidable', 'LaunchFwd', 'SetFallDistance', 'CreativeMode', 'AttackAnimation', 'DisplayHologram', ' SetAbsorption ', 'DeathDrops', 'ShowInv', 'SetFoodLevel', 'PlayerDisguise', 'SetSaturation', 'WeatherClear', 'SendTitle', 'Title', 'ScoreDefFormat', 'StopSound', 'HealthAttribute', 'ParticleLine']
ENTITY_ACTION_ACTION = Literal['DispRotationEuler', 'Shear', 'SetVelocity', 'SetGlowSquidDark', 'SetFrogType', 'DispRotAxisAngle', 'Damage', 'SetMobSitting', 'SendAnimation', 'DisableGlowing', 'SetWardenAnger', 'SetHorsePattern', 'Heal', 'SetPandaSadTicks', 'SetItemOwner', 'SetDyeColor', 'LaunchUp', 'SetAge', ' SetName ', 'NoGravity', 'SetArmsRaised', 'SetMoveSpeed', 'SetInvulnerable', 'SetFriction', 'ProjColl', 'ArmorStandTags', 'SetPickupDelay', 'DropItems', 'MiscAttribute', 'SetCreeperPower', 'SetMarker', 'RemoveCustomTag', 'SetAbsorption', 'CreeperCharged', 'SetFireTicks', 'CombatAttribute', 'SetName', 'Jump', 'BDisplayBlock', 'SetFreezeTicks', 'TDisplaySeeThru', 'SetGliding', 'SetRotation', 'SetPandaRolling', 'SetFishPattern', 'SetWolfType', 'DispInterpolation', 'SetHandItem', 'SetEndermanBlock', 'LaunchToward', 'SetArmor', ' SetArmor ', 'GetCustomTag', 'InteractionSize', 'FaceLocation', 'SetCatType', 'SetArrowDamage', 'DisplayScale', 'TDisplayAlign', 'LaunchProj', 'EnableAI', 'DisplayBillboard', 'NoProjColl', 'Tame', 'SetGoatScreaming', 'SetBeeStinger', 'MobDisguise', 'SetMinecartBlock', 'FoxSleeping', 'SetEquipment', 'SetSilenced', 'SetBeeNectar', 'AttachLead', 'RemovePotion', 'RemoveEffect', 'ShearSheep', 'ArmorStandSlots', 'SetAllayDancing', 'SetRabbitType', 'SetSize', 'ShowName', 'SetAngry', 'SetAngerTicks', 'Undisguise', 'SetDeathDrops', 'SetPersistent', 'ProjectileItem', 'SetNameColor', 'SetCarryingChest', 'SetParrotColor', 'DispTranslation', 'Remove', 'TDispBackground', 'DisplayCullingSize', 'HideName', 'SetAxolotlColor', 'GetAllEntityTags', 'SetAI', 'KBAttribute', 'MovementAttribute', 'SetRiptiding', 'SetArrowNoClip', 'FallingAttribute', 'SetProjSource', 'SetFoxLeaping', 'SetPandaGene', 'SetMaxHealth', 'SetFishingTime', 'EndCrystalBeam', 'FrogEat', 'DisplayBrightness', 'SetProfession', 'ClearPotions', 'ClearEffects', 'ArmorStandParts', ' SetNameVisible ', 'SetTarget', 'TDisplayShadow', 'SetNameVisible', 'SetInvulTicks', 'SetShulkerPeek', ' SetPose ', 'SetRearing', 'SetCloudRadius', 'SetGravity', 'DispTPDuration', 'SetWitherInvul', 'Silence', 'SetArrowPierce', 'DisplayShadow', 'InteractResponse', 'UseItem', 'RideEntity', 'DisplayMatrix', 'NoDrops', 'SnifferState', 'EnableGlowing', 'Teleport', 'DisplayGlowColor', 'SetVisualFire', 'SetAge/Size', 'L SetArmor', 'SetSaddle', 'SetBulletTarget', 'TDisplayLineWidth', 'SetDragonPhase', 'SetLlamaColor', 'SetVillagerBiome', 'SetCreeperFuse', 'SetBaby', 'MooshroomType', 'SetInvisible', 'SheepEat', 'SetCatResting', 'GivePotion', 'GiveEffect', 'SetGoatHorns', 'SetGlowing', 'SetPandaOnBack', 'IDisplayModelType', 'SetHealth', 'BlockDisguise', 'SetCollidable', 'ArmorStandPose', 'SetPose', 'LaunchFwd', 'SetFallDistance', 'MoveToLoc', 'TDisplayOpacity', 'IDisplayItem', 'AttackAnimation', 'SnowmanPumpkin', 'SetCustomTag', 'Gravity', 'DisplayViewRange', 'NoAI', 'PlayerDisguise', 'SetItem', 'Explode', 'ExplodeCreeper', 'SetDigging', 'MoveTo', 'SetArrowHitSound', 'SetVexCharging', 'SetVillagerExp', 'IgniteCreeper', 'SetCelebrating', 'TDisplayText', 'SetHorseJump', 'Unsilence', 'HealthAttribute', 'Ram', 'SetFoxType']
GAME_ACTION_ACTION = Literal['FillContainer', 'BreakBlock', 'L PFX Spiral', 'ParticleSphere', 'PFX Sphere', 'ChangeSign', 'WebRequest', 'ClearScBoard', 'HideSidebar', 'SpawnItemDisplay', 'WriteTransaction', 'ParticleSpiral', 'PFX Spiral', 'SetBlockData', 'Firework', 'SetEventDamage', 'SpawnItem', 'SignColor', 'ShulkerBullet', 'FireworkEffect', 'SetContainer', 'SpawnInteraction', 'SetItemInSlot', 'CloneRegion', 'CopyBlocks', 'UncancelEvent', 'SetLecternBook', 'SpawnArmorStand', 'SpawnBlockDisp', 'ClearContainer', 'CancelEvent', 'ParticleEffect', 'Particle FX', 'SpawnFangs', 'SetEventSound', 'SetEventXP', 'LockContainer', 'RemoveScore', 'CreateHologram', 'SetExhaustion', 'ParticleCircle', 'PFX Circle', 'PFX Line [A]', 'ClearItems', 'StartLoop', 'SetFurnaceSpeed', 'BlockDropsOn', 'BoneMeal', 'DebugStackTrace', 'FallingBlock', 'DiscordWebhook', 'TickBlock', 'ReplaceItems', 'SetEventProj', 'Explosion', 'SpawnMob', 'SetBrushableItem', 'ParticleLineA', 'SpawnEnderEye', 'ShowSidebar', 'SpawnPotionCloud', 'LaunchProj', ' SetBlock ', 'SpawnItemDisp', 'SetBlockGrowth', 'Wait', 'SetContainerName', 'SetHead', 'RemoveHologram', 'RemoveItems', 'SpawnRngItem', 'SetRegion', 'SetBlock', 'ParticleCircleA', 'PFX Circle [A]', 'SpawnTNT', 'SpawnExpOrb', 'SetBiome', 'SetEventHeal', 'PFX Path', 'ApplyTransaction', 'ParticleRay', 'PFX Ray', 'GenerateTree', 'StopLoop', 'SetScObj', 'SpawnCrystal', 'SetCampfireItem', 'SpawnTextDisplay', 'SpawnVehicle', 'Lightning', 'ParticleSpiralA', 'PFX Spiral [A]', 'SetScore', 'ParticleCluster', 'PFX Cluster', 'L PFX Cluster', 'BlockDropsOff', 'ParticleLine', 'PFX Line']
SET_VAR_ACTION = Literal['GetItemFood', 'String', 'Text', 'SetParticleType', 'SetItemEnchants', 'ClearItemTag', 'PurgeVars', 'ShiftAllAxes', 'ShiftLocCoord', 'GetParticleMat', 'SetParticleSprd', 'AbsoluteValue', 'RemoveItemAttrs', 'AppendValue', '%', 'ShiftOnVector', 'AddVectorLoc', 'GetItemAttribute', 'ClearDict', '+', 'ShiftRotation', 'RotateLocation', '-', ' GetItemName ', 'GetItemRarity', 'MultiplyVector', '/', 'GetSignText', 'Bitwise', 'GetLecternPage', 'ParseX', 'ShiftOnAxis', 'ShiftAxis', 'ParseY', 'VectorBetween', 'ParseZ', 'GetVectorComp', '=', 'RmText', 'AddItemAttribute', 'GetCenterLoc', 'FindCenter', 'Base64Decode', 'AlignLoc', 'GetSoundVolume', 'RandomNumber', 'GetCrossbowProj', 'GzipDecompress', 'ContainerName', 'Raycast', 'RotateAroundVec', 'SetItemFood', 'GetParticleMotion', 'SetParticleMotion', 'Average', 'WrapNumber', 'ClampLoc', 'SetY', 'SetMapTexture', 'GetBlockData', 'SetX', 'SortDict', 'GetLecternBook', 'GetCustomSound', 'CrossProduct', 'x', 'GetParticleRoll', 'ParseYaw', 'DotProduct', 'SetZ', 'SetArmorTrim', 'PopListValue', 'SetParticleOpac', 'Noise', 'MinNumber', 'MinValue', 'Min', 'GetPotionType', 'SetItemName', 'ListLength', 'Sine', 'DirectionName', 'RepeatString', 'DuplicateText', 'RepeatText', 'GetItemLore', 'JoinString', 'JoinText', 'ReverseList', 'DedupList', 'CreateDict', 'GetBlockByMCTag', 'RoundNumber', 'FaceLocation', 'GetItemLoreLine', 'SetVectorLength', 'JsonFromValue', 'SetPotionDur', 'BlockResistance', 'SplitString', 'SplitText', 'NormalRandom', 'SetPotionType', 'AlignVector', 'SetItemDura', 'SetBreakability', 'SetMaxAmount', ' GetSignText ', 'RaycastEntity', 'SetDictValue', 'SetAllCoords', 'RGBColor', 'SetCanDestroy', 'HSLColor', ' GetDirection ', ' GetItemLore ', 'RemoveListIndex', 'CellularNoise', 'Base64Encode', 'Logarithm', 'SetItemTag', 'TrimString', 'TrimText', 'ParseMiniMessageExpr', 'GetItemAmount', 'SetPotionAmp', 'GetCanDestroy', 'RotateAroundAxis', 'GetItemName', 'GetItemDura', 'ShiftInDirection', 'WrapNum', 'ReplaceString', 'ReplaceText', 'SetItemGlowing', ' SetItemName ', 'SetLodestoneLoc', 'FlattenList', 'BlockHardness', 'GetPotionAmp', 'GetParticleAmount', 'GetDictSize', 'SetItemAmount', 'SubtractVectors', 'SetCase', 'SetParticleColor', 'GetLight', ' GetBookText ', 'GetDictValues', 'Vector', 'Distance', 'SetItemLore', 'ParseMiniMessage', 'Root', 'SquareRoot', 'SetParticleAmount', 'AddItemEnchant', 'AddItemToolRule', 'GetItemType', 'GetDirection', 'GetLoreLine', 'ArcTangent2', 'GetParticleType', 'SetItemMaxDura', 'RemoveString', 'RemoveText', 'GetAllBlockData', 'SetCrossbowProj', 'MaxNumber', 'MaxValue', 'Max', 'GetDictKeys', 'TrimStyledText', 'SetParticleMat', 'GetCoord', 'RemoveItemTag', 'SetParticleSize', 'GetPotionDur', 'RandomLoc', 'SetSoundType', 'GetLodestoneLoc', 'ShiftDirection', 'GetContainerName', 'GetParticleSprd', 'ReflectVector', 'ClearItemAttrs', 'GetHeadOwner', 'GetItemEnchants', 'AppendDict', 'GetMaxItemAmount', 'GetColorChannels', ' SetDirection ', 'SetListValue', ' SetItemEnchants ', 'SetBookText', 'RandomValue', 'RandomObj', ' SetItemFlags ', 'SetItemType', 'GetSoundType', 'GetListValue', 'ValueToJson', 'BounceNum', 'Tangent', 'VoronoiNoise', 'SetDirection', 'FaceDirection', 'HSBColor', 'HSVColor', '+=', 'Interpolate', 'GetSoundVariant', 'GetItemColor', 'ClearFormatting', 'InsertListValue', 'InsertListIndex', 'SetSoundVolume', 'SetCoord', 'AddVectors', 'SanitizeTags', 'SetPitch', 'GetParticleFade', 'RaycastBlock', 'SetItemTool', ' GetItemEnchants ', 'SetHeadTexture', 'SetHeadOwner', 'GzipCompress', 'PerlinNoise', 'WorleyNoise', 'SetItemColor', 'SetLeatherColor', 'GetParticleColor', 'SetSoundPitch', ' RoundNumber ', 'GetCanPlaceOn', 'SortList', 'SetCustomSound', 'RemoveDictEntry', 'FormatTime', 'SetItemFlags', 'StringLength', 'TextLength', 'GetItemEffects', 'StyledText', 'GetMiniMessageExpr', 'SetYaw', ' SetItemLore ', 'SetItemEffects', '-=', 'GetItemTag', 'CreateList', 'AppendList', 'GetContainerItems', 'ShiftToward', 'ShiftTowards', 'ShiftLocTowards', 'TrimList', 'GradientNoise', 'SetItemHideTooltip', 'GetBlockDrops', 'ClearEnchants', 'Cosine', 'GetParticleOpac', 'GetItemByMCTag', 'SetParticleFade', 'SetVectorComp', 'ParseNumber', 'Exponent', 'ShiftAllDirs', 'GetValueIndex', 'RemItemEnchant', 'AddItemLore', 'GetBookText', 'SetParticleRoll', 'SetSoundVariant', 'ShiftLocation', 'RandomizeList', 'ClampNumber', 'Round', 'GetSoundPitch', 'TranslateColors', 'GetBlockGrowth', 'GetAllItemTags', 'RemoveListValue', 'BytesToString', 'ShiftAllDirections', 'ValueNoise', 'WebResponse', 'SetCanPlaceOn', 'GetBlockType', 'ParsePitch', 'GetDictValue', 'ContainerLock', 'GetBlockPower', 'GetVectorLength', 'ContentLength', 'SetModelData', 'StringToBytes', 'SetCoords', 'GetMaxAmount', 'GetParticleSize']
IF_PLAYER_ACTION = Literal['HasRoomForItem', 'PHasRoomForItem', 'IsHoldingOff', 'NoItemCooldown', 'IsUsingItem', 'HasAllItems', 'HasItem', 'BlockEquals', 'IsWearing', 'PIsWearing', 'IsNear', 'PIsNear', 'IsRiding', 'CmdEquals', 'CursorItem', 'SlotEquals', 'ItemEquals', 'MenuSlotEquals', 'HasPermission', ' IsRiding ', 'MainHandEquals', 'IsSneaking', 'IsFlying', 'InvOpen', 'HasSlotItem', 'IsSprinting', 'IsGliding', 'CmdArgEquals', 'IsLookingAt', 'InWorldBorder', 'IsInGameMode', 'UsingPack', 'IsSwimming', 'StandingOn', 'PStandingOn', ' StandingOn ', ' PStandingOn ', 'IsGrounded', 'OwnsProduct', 'IsHoldingMain', 'IsHolding', 'IsBlocking', 'IsHitboxNear', 'PHitboxIsNear', 'HasPotion', 'HasEffect', 'NameEquals', 'PNameEquals', 'CanFly']
IF_ENTITY_ACTION = Literal['IsVehicle', 'IsGrounded', 'EIsGrounded', 'IsType', 'IsProj', 'IsMob', 'HasCustomTag', 'IsSheared', 'IsItem', ' IsRiding ', 'Exists', 'IsHitboxNear', 'EHitboxIsNear', 'IsNear', 'EIsNear', 'HasPotion', 'IsRiding', 'StandingOn', 'EStandingOn', 'NameEquals', 'ENameEquals', ' StandingOn ', ' EStandingOn ']
IF_GAME_ACTION = Literal['SignHasTxt', 'HasRoomForItem', 'EventBlockEquals', 'CommandEquals', 'EventItemEquals', ' SignHasTxt ', 'AttackIsCrit', 'ContainerHas', 'BlockEquals', 'GBlockEquals', 'InBlock', 'BlockPowered', 'HasPlayer', 'ContainerHasAll', 'CmdArgEquals', 'EventCancelled', 'IsChunkLoaded']
IF_VAR_ACTION = Literal['<=', 'ItemHasEnchant', 'ItemIsBlock', 'DictValueEquals', 'ItemHasTag', 'ListSizeEquals', 'ListIsEmpty', 'StringMatches', ' TextMatches ', 'StartsWith', 'ListValueEq', 'VarIsType', 'TextMatches', 'EqIgnoreCase', 'IsNear', 'VIsNear', ' InRange ', 'VarExists', 'BlockIsSolid', 'ItemEquals', 'VItemEquals', 'ListContains', 'InRange', 'LocIsNear', 'Contains', '!=', '<', '=', ' = ', '>', 'EndsWith', '>=', 'VarIsEmpty', 'DictHasKey']
REPEAT_ACTION = Literal['Adjacent', 'Path', 'Multiple', 'N Times', 'Grid', 'While', 'WhileCond', 'Range', 'ForEach', 'Sphere', 'Forever', ' Range ', 'ForEachEntry']
SELECT_OBJ_ACTION = Literal['LastMob', 'RandomPlayer', 'LastEntity', 'Shooter', 'AllMobs', 'EntityName', 'FilterRandom', 'RandomSelected', 'DefaultEntity', 'PlayerName', 'AllEntities', 'Damager', 'FilterDistance', 'FilterRay', 'Reset', 'None', 'EventTarget', 'Killer', 'Victim', 'EntitiesCond', 'AllPlayers', 'Invert', 'RandomEntity', 'FilterCondition', 'FilterSelect', 'MobsCond', 'FilterSort', 'Projectile', 'DefaultPlayer', 'PlayersCond', 'MobName']
CONTROL_ACTION = Literal['StopRepeat', 'Return', 'ReturnNTimes', 'Skip', 'End', 'Wait']
EVENT_ACTION = Literal['ClickContainerSlot', 'CloseInv', 'StartFly', 'BreakBlock', 'StartSprint', 'MobKillPlayer', 'Teleport', 'ShootBow', 'ChangeSign', 'StopFly', 'TameEntity', 'LeftClick', 'PlayerTakeDmg', 'ProjHit', 'KillPlayer', 'VehicleJump', 'ClickInvSlot', 'ClickOwnInv', 'Respawn', 'SwapHands', 'PackLoad', 'DamageEntity', 'Sneak', 'PlayerHeal', 'ClickPlayer', 'Consume', 'StartGlide', 'Death', 'PlaceBlock', 'Walk', 'PickUpItem', 'PickupItem', 'Dismount', 'CloudImbuePlayer', 'Leave', 'Quit', 'DropItem', 'LeftClickPlayer', 'ChangeSlot', 'ClickEntity', 'HorseJump', 'ShootProjectile', 'Move', 'Unsneak', 'PlayerCombust', 'Chat', 'Fish', 'FishEvent', 'FallDamage', 'BreakItem', 'LoopEvent', 'PlayerResurrect', 'RightClick', 'Purchase', 'ClickMenuSlot', 'ClickItem', 'Riptide', 'StopGlide', 'KillMob', 'Join', 'EntityDmgPlayer', 'StopSprint', 'Jump', 'LoadCrossbow', 'ProjDmgPlayer', 'Command', 'LeftClickEntity', 'Exhaustion', 'PackDecline', 'PlayerDmgPlayer']
ENTITY_EVENT_ACTION = Literal['EntityKillEntity', 'EntityCombust', 'BlockFall', 'ProjKillEntity', 'EntityDmgEntity', 'FallingBlockLand', 'EntityResurrect', 'ItemMerge', 'EntityHeal', 'Teleport', 'ShootBow', 'EntityDmg', 'ProjDmgEntity', 'EntityExplode', 'EntityDeath', 'VehicleDamage', 'Transform', 'RegrowWool']
GAME_VALUE_NAME = Literal['Current Health', 'Maximum Health', 'Absorption Health', 'Food Level', 'Food Saturation', 'Food Exhaustion', 'Attack Damage', 'Attack Speed', 'Attack Cooldown', 'Attack Cooldown Ticks', 'Armor Points', 'Armor Toughness', 'Invulnerability Ticks', 'Experience Level', 'Experience Progress', 'Fire Ticks', 'Freeze Ticks', 'Remaining Air', 'Fall Distance', 'Held Slot', 'Ping', 'Steer Sideways Movement', 'Steer Forward Movement', 'Item Usage Progress', 'Flight Speed', 'Walk Speed', 'Location', 'Target Block Location', 'Target Block Side', 'Eye Location', 'X-Coordinate', 'Y-Coordinate', 'Z-Coordinate', 'Midpoint Location', 'Pitch', 'Yaw', 'Body Yaw', 'Standing Block Location', 'Spawn Location', 'Velocity', 'Direction', 'Entity Width', 'Entity Height', 'Main Hand Item', 'Off Hand Item', 'Armor Items', 'Hotbar Items', 'Inventory Items', 'Cursor Item', 'Inventory Menu Items', 'Saddle Item', 'Entity Item', 'Name ', 'UUID', 'Entity Type', 'Game Mode', 'Open Inventory Title ', 'Potion Effects', 'Vehicle', 'Passengers ', 'Lead Holder', 'Attached Leads', 'Targeted Entity UUID', 'Projectile Shooter UUID', 'Display Entity Translation', 'Display Entity Scale', 'Pose', 'Event Block Location', 'Event Block Side', 'Event Damage', 'Damage Event Cause', 'Raw Event Damage', 'Event Death Message', 'Event Heal Amount', 'Heal Event Cause', 'Event Explosion Affected Blocks', 'Event Power', 'Event Command', 'Event Command Arguments', 'Event Item', 'Event Hotbar Slot', 'Event Clicked Slot Index', 'Event Clicked Slot Item', 'Event Clicked Slot New Item', 'Close Inventory Event Cause', 'Inventory Event Click Type', 'Fish Event Cause', 'Teleport Event Cause', 'Teleport Location', 'Exhaustion Event Cause', 'Event Exhaustion', 'Transform Event Cause', 'Event Transform Entities', 'Event Hit Type', 'Product ID', 'Event Message', 'Event Sign Text', 'Event Sign Side', 'Combust Event Duration', 'Combust Event Cause', 'Player Count', 'CPU Usage', 'Server TPS', 'Timestamp', 'Selection Size', 'Selection Target Names', 'Selection Target UUIDs', 'Plot ID', 'Plot Name', 'Plot Size', 'Microseconds Since Startup', 'Ticks Since Startup', 'Active Block Transactions', 'Plot Player Names', 'Plot Player UUIDs']
SOUND_NAME = Literal['Axolotl Attack', 'Axolotl Death', 'Axolotl Hurt', 'Axolotl Ambient Air', 'Axolotl Ambient Water', 'Axolotl Splash', 'Axolotl Swim', 'Bat Ambient', 'Bat Death', 'Bat Fly', 'Bat Hurt', 'Bat Takeoff', 'Bee Death', 'Bee Hurt', 'Bee Loop', 'Bee Loop (Aggressive)', 'Bee Pollinate', 'Bee Sting', 'Camel Ambient', 'Camel Dash Ready', 'Camel Dash', 'Camel Death', 'Camel Eat', 'Camel Hurt', 'Camel Equip Saddle', 'Camel Sit', 'Camel Stand', 'Camel Step', 'Camel Step on Sand', 'Cat Ambient', 'Cat Beg For Food', 'Cat Death', 'Cat Eat', 'Cat Hiss', 'Cat Hurt', 'Cat Purr', 'Cat Purreow', 'Stray Cat Ambient', 'Chicken Ambient', 'Chicken Death', 'Chicken Hurt', 'Chicken Lay Egg', 'Chicken Step', 'Cow Ambient', 'Cow Death', 'Cow Hurt', 'Cow Milk', 'Cow Step', 'Dolphin Ambient', 'Dolphin Ambient (Water)', 'Dolphin Attack', 'Dolphin Death', 'Dolphin Eat', 'Dolphin Hurt', 'Dolphin Jump', 'Dolphin Play', 'Dolphin Splash', 'Dolphin Swim', 'Donkey Ambient', 'Donkey Angry', 'Donkey Equip Chest', 'Donkey Death', 'Donkey Eat', 'Donkey Hurt', 'Donkey Jump', 'Fish Swim', 'Cod Ambient', 'Cod Death', 'Cod Flop', 'Cod Hurt', 'Salmon Ambient', 'Salmon Death', 'Salmon Flop', 'Salmon Hurt', 'Tropical Fish Ambient', 'Tropical Fish Death', 'Tropical Fish Flop', 'Tropical Fish Hurt', 'Fox Aggravate', 'Fox Ambient', 'Fox Bite', 'Fox Death', 'Fox Eat', 'Fox Hurt', 'Fox Screech', 'Fox Sleep', 'Fox Sniff', 'Fox Spit', 'Fox Teleport', 'Goat Ambient', 'Goat Death', 'Goat Eat', 'Goat Hurt', 'Goat Long Jump', 'Goat Milk', 'Goat Prepare Ram', 'Goat Ram Impact', 'Goat Horn Break', 'Goat Step', 'Screaming Goat Ambient', 'Screaming Goat Death', 'Screaming Goat Eat', 'Screaming Goat Hurt', 'Screaming Goat Long Jump', 'Screaming Goat Milk', 'Screaming Goat Prepare Ram', 'Screaming Goat Ram Impact', 'Glow Squid Ambient', 'Glow Squid Death', 'Glow Squid Hurt', 'Glow Squid Squirt', 'Horse Ambient', 'Horse Angry', 'Horse Breathe', 'Horse Death', 'Horse Eat', 'Horse Equip Saddle', 'Horse Equip Armor', 'Horse Gallop', 'Horse Hurt', 'Horse Jump', 'Horse Land', 'Horse Step', 'Horse Step (Wood)', 'Iron Golem Attack', 'Iron Golem Damage', 'Iron Golem Death', 'Iron Golem Hurt', 'Iron Golem Repair', 'Iron Golem Step', 'Llama Ambient', 'Llama Angry', 'Llama Death', 'Llama Eat', 'Llama Equip Chest', 'Llama Hurt', 'Llama Step', 'Llama Spit', 'Llama Swag', 'Mooshroom Convert', 'Mooshroom Eat', 'Mooshroom Milk', 'Mooshroom Shear', 'Brown Mooshroom Milk', 'Mule Angry', 'Mule Equip Chest', 'Mule Ambient', 'Mule Death', 'Mule Eat', 'Mule Hurt', 'Mule Jump', 'Ocelot Ambient', 'Ocelot Death', 'Ocelot Hurt', 'Panda Ambient', 'Panda Ambient (Aggressive)', 'Panda Ambient (Worried)', 'Panda Bite', 'Panda Sterile', 'Panda Death', 'Panda Eat', 'Panda Hurt', 'Panda Pre-Sneeze', 'Panda Sneeze', 'Panda Step', 'Parrot Ambient', 'Parrot Death', 'Parrot Hurt', 'Parrot Step', 'Parrot Eat', 'Parrot Fly', 'Parrot Imitate Blaze', 'Parrot Imitate Bogged', 'Parrot Imitate Breeze', 'Parrot Imitate Creeper', 'Parrot Imitate Drowned', 'Parrot Imitate Elder Guardian', 'Parrot Imitate Ender Dragon', 'Parrot Imitate Endermite', 'Parrot Imitate Evoker', 'Parrot Imitate Ghast', 'Parrot Imitate Guardian', 'Parrot Imitate Hoglin', 'Parrot Imitate Husk', 'Parrot Imitate Illusioner', 'Parrot Imitate Magma Cube', 'Parrot Imitate Phantom', 'Parrot Imitate Piglin', 'Parrot Imitate Pillager', 'Parrot Imitate Ravager', 'Parrot Imitate Piglin Brute', 'Parrot Imitate Shulker', 'Parrot Imitate Silverfish', 'Parrot Imitate Skeleton', 'Parrot Imitate Slime', 'Parrot Imitate Spider', 'Parrot Imitate Stray', 'Parrot Imitate Vex', 'Parrot Imitate Vindicator', 'Parrot Imitate Warden', 'Parrot Imitate Witch', 'Parrot Imitate Wither', 'Parrot Imitate Wither Skeleton', 'Parrot Imitate Zoglin', 'Parrot Imitate Zombie', 'Parrot Imitate Zombie Villager', 'Pig Ambient', 'Pig Death', 'Pig Equip Saddle', 'Pig Hurt', 'Pig Step', 'Polar Bear Ambient', 'Polar Bear Death', 'Polar Bear Hurt', 'Polar Bear Step', 'Polar Bear Warn', 'Baby Polar Bear Ambient', 'Pufferfish Ambient', 'Pufferfish Death', 'Pufferfish Deflate', 'Pufferfish Flop', 'Pufferfish Hurt', 'Pufferfish Inflate', 'Pufferfish Sting', 'Rabbit Ambient', 'Rabbit Death', 'Rabbit Hurt', 'Rabbit Jump', 'Killer Rabbit Attack', 'Sheep Ambient', 'Sheep Death', 'Sheep Hurt', 'Sheep Shear', 'Sheep Step', 'Sniffer Ambient', 'Sniffer Death', 'Sniffer Digging', 'Sniffer Stop Digging', 'Sniffer Drop Seed', 'Sniffer Eat', 'Sniffer Happy', 'Sniffer Hurt', 'Sniffer Scenting', 'Sniffer Searching', 'Sniffer Sniffing', 'Sniffer Step', 'Snow Golem Ambient', 'Snow Golem Death', 'Snow Golem Hurt', 'Snow Golem Shear', 'Snow Golem Shoot', 'Squid Ambient', 'Squid Death', 'Squid Hurt', 'Squid Squirt', 'Tadpole Death', 'Tadpole Hurt', 'Tadpole Flop', 'Tadpole Grow Up', 'Turtle Ambient', 'Turtle Death', 'Turtle Hurt', 'Turtle Shamble', 'Turtle Lay Egg', 'Turtle Egg Crack', 'Turtle Egg Hatch', 'Turtle Egg Break', 'Turtle Swim', 'Baby Turtle Death', 'Baby Turtle Hurt', 'Baby Turtle Shamble', 'Wolf Ambient', 'Wolf Death', 'Wolf Growl', 'Wolf Howl', 'Wolf Hurt', 'Wolf Pant', 'Wolf Shake', 'Wolf Step', 'Wolf Whine', 'Allay Ambient (Without Item)', 'Allay Ambient (With Item)', 'Allay Death', 'Allay Hurt', 'Allay Item Given', 'Allay Item Pickup', 'Allay Item Thrown', 'Armadillo Ambient', 'Armadillo Brush', 'Armadillo Death', 'Armadillo Eat', 'Armadillo Hurt', 'Armadillo Hurt (Reduced)', 'Armadillo Land', 'Armadillo Peek', 'Armadillo Roll', 'Armadillo Scute Drop', 'Armadillo Step', 'Armadillo Unroll Finish', 'Armadillo Unroll Start', 'Bogged Ambient', 'Bogged Death', 'Bogged Hurt', 'Bogged Shear', 'Bogged Step', 'Breeze Charge', 'Breeze Death', 'Breeze Deflect', 'Breeze Hurt', 'Breeze Idle (Air)', 'Breeze Idle (Ground)', 'Breeze Inhale', 'Breeze Jump', 'Breeze Land', 'Breeze Shoot', 'Breeze Slide', 'Breeze Whirl', 'Breeze Wind Burst', 'Frog Ambient', 'Frog Death', 'Frog Eat', 'Frog Hurt', 'Frog Step', 'Frog Lay Spawn', 'Frog Long Jump', 'Frog Tongue', 'Strider Ambient', 'Strider Death', 'Strider Eat', 'Strider Equip Saddle', 'Strider Happy', 'Strider Hurt', 'Strider Retreat', 'Strider Step', 'Strider Step (Lava)', 'Villager Ambient', 'Villager Death', 'Villager Hurt', 'Villager No', 'Villager Trading', 'Villager Yes', 'Armorer Work', 'Butcher Work', 'Cartographer Work', 'Cleric Work', 'Farmer Work', 'Fisherman Work', 'Fletcher Work', 'Leatherworker Work', 'Librarian Work', 'Mason Work', 'Shepherd Work', 'Toolsmith Work', 'Weaponsmith Work', 'Villager Celebrate', 'Wandering Trader Ambient', 'Wandering Trader Death', 'Wandering Trader Disappear', 'Wandering Trader Drink Milk', 'Wandering Trader Drink Potion', 'Wandering Trader Hurt', 'Wandering Trader No', 'Wandering Trader Reappear', 'Wandering Trader Trading', 'Wandering Trader Yes', 'Creeper Death', 'Explode', 'Creeper Hurt', 'Creeper Primed', 'Drowned Ambient', 'Drowned Ambient (Land)', 'Drowned Death', 'Drowned Death (Land)', 'Drowned Hurt', 'Drowned Hurt (Land)', 'Drowned Shoot', 'Drowned Step', 'Drowned Swim', 'Elder Guardian Ambient', 'Elder Guardian Ambient (Land)', 'Elder Guardian Curse', 'Elder Guardian Death', 'Elder Guardian Death (Land)', 'Elder Guardian Flop', 'Elder Guardian Hurt', 'Elder Guardian Hurt (Land)', 'Guardian Ambient', 'Guardian Ambient (Land)', 'Guardian Attack', 'Guardian Death', 'Guardian Death (Land)', 'Guardian Flop', 'Guardian Hurt', 'Guardian Hurt (Land)', 'Husk Ambient', 'Husk Death', 'Husk Hurt', 'Husk Step', 'Husk Drown', 'Phantom Ambient', 'Phantom Bite', 'Phantom Death', 'Phantom Flap', 'Phantom Hurt', 'Phantom Swoop', 'Ravager Ambient', 'Ravager Attack', 'Ravager Death', 'Ravager Hurt', 'Ravager Roar', 'Ravager Step', 'Ravager Stunned', 'Ravager Celebrate', 'Silverfish Ambient', 'Silverfish Death', 'Silverfish Hurt', 'Silverfish Step', 'Skeleton Ambient', 'Skeleton Convert To Stray', 'Skeleton Death', 'Skeleton Hurt', 'Skeleton Step', 'Skeleton Shoot', 'Skeleton Horse Ambient', 'Skeleton Horse Ambient (Water)', 'Skeleton Horse Death', 'Skeleton Horse Gallop (Water)', 'Skeleton Horse Hurt', 'Skeleton Horse Jump (Water)', 'Skeleton Horse Step (Water)', 'Skeleton Horse Swim', 'Slime Attack', 'Slime Death', 'Slime Hurt', 'Slime Jump', 'Slime Squish', 'Small Slime Death', 'Small Slime Hurt', 'Small Slime Jump', 'Small Slime Squish', 'Spider Ambient', 'Spider Death', 'Spider Hurt', 'Spider Step', 'Stray Ambient', 'Stray Death', 'Stray Hurt', 'Stray Step', 'Vex Ambient', 'Vex Charge', 'Vex Death', 'Vex Hurt', 'Warden Ambient', 'Warden Death', 'Warden Hurt', 'Warden Step', 'Warden Groan', 'Warden Angry', 'Warden Attack', 'Warden Dig', 'Warden Emerge', 'Warden Heartbeat', 'Warden Listening', 'Warden Listening (Angry)', 'Warden Approach', 'Warden Approach (Closer)', 'Warden Approach (Closest)', 'Warden Roar', 'Warden Sniff', 'Warden Sonic Boom', 'Warden Sonic Charge', 'Warden Tendril Click', 'Witch Ambient', 'Witch Death', 'Witch Drink', 'Witch Hurt', 'Witch Throw', 'Witch Celebrate', 'Zombie Ambient', 'Zombie Attack Wooden Door', 'Zombie Attack Iron Door', 'Zombie Break Wooden Door', 'Zombie Death', 'Zombie Destroy Egg', 'Zombie Drown', 'Zombie Infect', 'Zombie Hurt', 'Zombie Step', 'Zombie Horse Ambient', 'Zombie Horse Death', 'Zombie Horse Hurt', 'Zombie Villager Ambient', 'Zombie Villager Convert', 'Zombie Villager Cure', 'Zombie Villager Death', 'Zombie Villager Hurt', 'Zombie Villager Step', 'Blaze Ambient', 'Blaze Burn', 'Blaze Death', 'Blaze Hurt', 'Blaze Shoot', 'Ghast Ambient', 'Ghast Death', 'Ghast Hurt', 'Ghast Scream', 'Ghast Shoot', 'Ghast Warn', 'Hoglin Ambient', 'Hoglin Angry', 'Hoglin Attack', 'Hoglin Zombify', 'Hoglin Death', 'Hoglin Hurt', 'Hoglin Retreat', 'Hoglin Step', 'Magma Cube Death', 'Magma Cube Hurt', 'Magma Cube Jump', 'Magma Cube Squish', 'Small Magma Cube Death', 'Small Magma Cube Hurt', 'Small Magma Cube Squish', 'Piglin Admiring Item', 'Piglin Ambient', 'Piglin Angry', 'Piglin Celebrate', 'Piglin Zombify', 'Piglin Death', 'Piglin Hurt', 'Piglin Jealous', 'Piglin Retreat', 'Piglin Step', 'Piglin Brute Ambient', 'Piglin Brute Angry', 'Piglin Brute Zombify', 'Piglin Brute Death', 'Piglin Brute Hurt', 'Piglin Brute Step', 'Wither Ambient', 'Wither Break Block', 'Wither Death', 'Wither Hurt', 'Wither Shoot', 'Wither Spawn', 'Wither Skeleton Ambient', 'Wither Skeleton Death', 'Wither Skeleton Hurt', 'Wither Skeleton Step', 'Zoglin Ambient', 'Zoglin Angry', 'Zoglin Attack', 'Zoglin Death', 'Zoglin Hurt', 'Zoglin Step', 'Zombified Piglin Ambient', 'Zombified Piglin Angry', 'Zombified Piglin Death', 'Zombified Piglin Hurt', 'Enderman Ambient', 'Enderman Death', 'Enderman Hurt', 'Enderman Scream', 'Enderman Stare', 'Enderman Teleport', 'Endermite Ambient', 'Endermite Death', 'Endermite Hurt', 'Endermite Step', 'Ender Dragon Ambient', 'Ender Dragon Death', 'Ender Dragon Hurt', 'Ender Dragon Fireball Explode', 'Ender Dragon Flap', 'Ender Dragon Growl', 'Ender Dragon Shoot', 'Shulker Ambient', 'Shulker Close', 'Shulker Death', 'Shulker Hurt', 'Shulker Hurt (Closed)', 'Shulker Open', 'Shulker Shoot', 'Shulker Teleport', 'Shulker Bullet Explode', 'Shulker Bullet Break', 'Evoker Ambient', 'Evoker Cast Spell', 'Evoker Death', 'Evoker Hurt', 'Evoker Prepare Attack', 'Evoker Prepare Summon', 'Evoker Prepare Wololo', 'Evoker Fangs Attack', 'Evoker Celebrate', 'Illusioner Ambient', 'Illusioner Death', 'Illusioner Hurt', 'Illusioner Cast Spell', 'Illusioner Mirror Move', 'Illusioner Prepare Blindness', 'Illusioner Prepare Mirror', 'Pillager Ambient', 'Pillager Death', 'Pillager Hurt', 'Pillager Celebrate', 'Vindicator Ambient', 'Vindicator Death', 'Vindicator Hurt', 'Vindicator Celebrate', 'Player Attack (Critical)', 'Player Attack (Knockback)', 'Player Attack (No Damage)', 'Player Attack (Strong)', 'Player Attack (Sweep)', 'Player Attack (Weak)', 'Player Breathe', 'Player Burn', 'Player Freeze', 'Player Burp', 'Player Drown', 'Player Hurt (Sweet Berry Bush)', 'Player Level Up', 'Player Splash (High Speed)', 'Death', 'Drink', 'Eat', 'Extinguish', 'Fall', 'Fall (Big)', 'Hurt', 'Splash', 'Swim', 'Challenge Complete', 'UI Button Click', 'UI Toast In', 'UI Toast Out', 'Arrow Hit', 'Arrow Hit Player', 'Arrow Shoot', 'Egg Throw', 'Ender Eye Death', 'Ender Eye Launch', 'Enderpearl Throw', 'Experience Bottle Throw', 'Lingering Potion Throw', 'Splash Potion Break', 'Splash Potion Throw', 'Snowball Throw', 'Firework Blast', 'Firework Blast Far', 'Firework Large Blast', 'Firework Large Blast Far', 'Firework Launch', 'Firework Shoot', 'Firework Twinkle', 'Firework Twinkle Far', 'Glow Item Frame Add Item', 'Glow Item Frame Break', 'Glow Item Frame Place', 'Glow Item Frame Remove Item', 'Glow Item Frame Rotate Item', 'Item Frame Add Item', 'Item Frame Break', 'Item Frame Place', 'Item Frame Remove Item', 'Item Frame Rotate Item', 'Painting Break', 'Painting Place', 'Boat Paddle Land', 'Boat Paddle Water', 'Minecart Inside', 'Minecart Inside Water', 'Rolling Minecart', 'Armor Stand Break', 'Armor Stand Fall', 'Armor Stand Hit', 'Armor Stand Place', 'Burn', 'Experience Orb Pickup', 'Leash Knot Break', 'Leash Knot Place', 'TNT Primed', 'Bad Omen Mob Effect', 'Raid Omen Mob Effect', 'Trial Omen Mob Effect', 'Creative Music', 'Credits Music', 'Boss Music', 'End Music', 'Menu Music', 'Overworld Music', 'Dripstone Caves Music', 'Frozen Peaks Music', 'Grove Music', 'Jagged Peaks Music', 'Lush Caves Music', 'Meadow Music', 'Snowy Slopes Music', 'Stony Peaks Music', 'Forest Music', 'Jungle Music', 'Sparse Jungle Music', 'Bamboo Jungle Music', 'Old Growth Taiga Music', 'Swamp Music', 'Desert Music', 'Badlands Music', 'Flower Forest Music', 'Cherry Grove Music', 'Deep Dark Music', 'Basalt Deltas Music', 'Crimson Forest Music', 'Nether Wastes Music', 'Soul Sand Valley Music', 'Underwater Music', 'Music Disc - 5', 'Music Disc - 11', 'Music Disc - 13', 'Music Disc - Blocks', 'Music Disc - Cat', 'Music Disc - Chirp', 'Music Disc - Creator', 'Music Disc - Creator (Music Box)', 'Music Disc - Far', 'Music Disc - Mall', 'Music Disc - Mellohi', 'Music Disc - Pigstep', 'Music Disc - Stal', 'Music Disc - Strad', 'Music Disc - Ward', 'Music Disc - Wait', 'Music Disc - Otherside', 'Music Disc - Precipice', 'Music Disc - Relic', 'Shield Block', 'Shield Break', 'Crossbow Hit', 'Crossbow Loading (End)', 'Crossbow Loading (Middle)', 'Crossbow Loading (Start)', 'Crossbow Quick Charge (I)', 'Crossbow Quick Charge (II)', 'Crossbow Quick Charge (III)', 'Crossbow Shoot', 'Mace Smash Air', 'Mace Smash Ground', 'Mace Smash Ground (Heavy)', 'Armor Equip', 'Armor Equip (Elytra)', 'Armor Equip (Leather)', 'Armor Equip (Chain)', 'Armor Equip (Iron)', 'Armor Equip (Gold)', 'Armor Equip (Diamond)', 'Armor Equip (Netherite)', 'Armor Equip (Turtle)', 'Elytra Flying', 'Thorns Hit', 'Wolf Armor Equip', 'Wolf Armor Unequip', 'Wolf Armor Break', 'Wolf Armor Crack', 'Wolf Armor Damage', 'Wolf Armor Repair', 'Trident Throw', 'Trident Hit', 'Trident Hit Ground', 'Trident Return', 'Trident Riptide (I)', 'Trident Riptide (II)', 'Trident Riptide (III)', 'Trident Thunder', 'Fire Charge Use', 'Flint and Steel Use', 'Axe Scrape', 'Axe Strip', 'Wax Off', 'Hoe Till', 'Shovel Flatten', 'Fishing Bobber Retrieve', 'Fishing Bobber Splash', 'Fishing Bobber Throw', 'Brushing (Generic)', 'Brushing Gravel', 'Brushing Gravel Complete', 'Brushing Sand', 'Brushing Sand Complete', 'Bottle Empty', 'Bottle Fill', 'Dragon Breath Fill', 'Honey Bottle Drink', 'Water Bucket Empty', 'Water Bucket Fill', 'Axolotl Bucket Empty', 'Axolotl Bucket Fill', 'Fish Bucket Empty', 'Fish Bucket Fill', 'Tadpole Bucket Empty', 'Tadpole Bucket Fill', 'Lava Bucket Empty', 'Lava Bucket Fill', 'Powder Bucket Empty', 'Powder Bucket Fill', 'Item Break', 'Item Pickup', 'Book Page Turn', 'Book Put in Lectern', 'Bone Meal Use', 'Chorus Fruit Teleport', 'Dye Use', 'Ink Sac Use', 'Glow Ink Sac Use', 'Goat Horn (Ponder)', 'Goat Horn (Sing)', 'Goat Horn (Seek)', 'Goat Horn (Feel)', 'Goat Horn (Admire)', 'Goat Horn (Call)', 'Goat Horn (Yearn)', 'Goat Horn (Dream)', 'Spyglass Use', 'Spyglass Stop Using', 'Bundle Drop Contents', 'Bundle Remove One', 'Bundle Item Insert', 'Totem Use', 'Wax Copper', 'Wind Charge Throw', 'Wind Charge Burst', 'Ominous Bottle Dispose', 'Bamboo Break', 'Bamboo Fall', 'Bamboo Hit', 'Bamboo Place', 'Bamboo Sapling Break', 'Bamboo Sapling Hit', 'Bamboo Sapling Place', 'Bamboo Step', 'Bamboo Planks Break', 'Bamboo Planks Fall', 'Bamboo Planks Hit', 'Bamboo Planks Place', 'Bamboo Planks Step', 'Beehive Drip', 'Beehive Enter', 'Beehive Exit', 'Beehive Shear', 'Beehive Work', 'Bone Block Break', 'Bone Block Fall', 'Bone Block Hit', 'Bone Block Place', 'Bone Block Step', 'Bubble Column Bubble Pop', 'Bubble Column Ambient (Up)', 'Bubble Column Inside (Up)', 'Bubble Column Ambient (Down)', 'Bubble Column Inside (Down)', 'Cherry Leaves Break', 'Cherry Leaves Fall', 'Cherry Leaves Hit', 'Cherry Leaves Place', 'Cherry Leaves Step', 'Cherry Sapling Break', 'Cherry Sapling Fall', 'Cherry Sapling Hit', 'Cherry Sapling Place', 'Cherry Sapling Step', 'Cherry Wood Break', 'Cherry Wood Fall', 'Cherry Wood Hit', 'Cherry Wood Place', 'Cherry Wood Step', 'Cobweb Break', 'Cobweb Fall', 'Cobweb Hit', 'Cobweb Place', 'Cobweb Step', 'Copper Bulb Break', 'Copper Bulb Fall', 'Copper Bulb Hit', 'Copper Bulb Place', 'Copper Bulb Step', 'Copper Bulb Turn Off', 'Copper Bulb Turn On', 'Copper Door Close', 'Copper Door Open', 'Copper Grate Break', 'Copper Grate Fall', 'Copper Grate Hit', 'Copper Grate Place', 'Copper Grate Step', 'Copper Trapdoor Close', 'Copper Trapdoor Open', 'Crafter Craft', 'Crafter Fail', 'Coral Block Break', 'Coral Block Fall', 'Coral Block Hit', 'Coral Block Place', 'Coral Block Step', 'Crop Plant', 'Crop Break', 'Plant Stop Growing', 'Fire Ambient', 'Fire Extinguish', 'Grass Break', 'Grass Fall', 'Grass Hit', 'Grass Place', 'Grass Step', 'Gravel Break', 'Gravel Fall', 'Gravel Hit', 'Gravel Place', 'Gravel Step', 'Lava Ambient', 'Lava Extinguish', 'Lava Pop', 'Lily Pad Place', 'Pumpkin Carve', 'Pink Petals Break', 'Pink Petals Fall', 'Pink Petals Hit', 'Pink Petals Place', 'Pink Petals Step', 'Powder Snow Break', 'Powder Snow Fall', 'Powder Snow Hit', 'Powder Snow Place', 'Powder Snow Step', 'Rooted Dirt Break', 'Rooted Dirt Fall', 'Rooted Dirt Hit', 'Rooted Dirt Place', 'Rooted Dirt Step', 'Sand Break', 'Sand Fall', 'Sand Hit', 'Sand Place', 'Sand Step', 'Suspicious Sand Break', 'Suspicious Sand Fall', 'Suspicious Sand Hit', 'Suspicious Sand Place', 'Suspicious Sand Step', 'Suspicious Gravel Break', 'Suspicious Gravel Fall', 'Suspicious Gravel Hit', 'Suspicious Gravel Place', 'Suspicious Gravel Step', 'Sniffer Egg Plop', 'Sniffer Egg Crack', 'Sniffer Egg Hatch', 'Snow Break', 'Snow Fall', 'Snow Hit', 'Snow Place', 'Snow Step', 'Stone Break', 'Stone Fall', 'Stone Hit', 'Stone Place', 'Stone Step', 'Sweet Berry Bush Break', 'Sweet Berry Bush Pick', 'Sweet Berry Bush Place', 'Seagrass Break', 'Seagrass Fall', 'Seagrass Hit', 'Seagrass Place', 'Seagrass Step', 'Vine Break', 'Vine Fall', 'Vine Hit', 'Vine Place', 'Vine Climb', 'Water Ambient', 'Water Enter', 'Water Exit', 'Wood Break', 'Wood Fall', 'Wood Hit', 'Wood Place', 'Wood Step', 'Nether Wood Break', 'Nether Wood Fall', 'Nether Wood Hit', 'Nether Wood Place', 'Nether Wood Step', 'Froglight Break', 'Froglight Fall', 'Froglight Hit', 'Froglight Place', 'Froglight Step', 'Frogspawn Break', 'Frogspawn Fall', 'Frogspawn Hit', 'Frogspawn Place', 'Frogspawn Step', 'Frogspawn Hatch', 'Mangrove Roots Break', 'Mangrove Roots Fall', 'Mangrove Roots Hit', 'Mangrove Roots Place', 'Mangrove Roots Step', 'Muddy Mangrove Roots Break', 'Muddy Mangrove Roots Fall', 'Muddy Mangrove Roots Hit', 'Muddy Mangrove Roots Place', 'Muddy Mangrove Roots Step', 'Mud Break', 'Mud Fall', 'Mud Hit', 'Mud Place', 'Mud Step', 'Amethyst Block Break', 'Amethyst Block Chime', 'Amethyst Block Fall', 'Amethyst Block Hit', 'Amethyst Block Place', 'Amethyst Block Step', 'Amethyst Cluster Break', 'Amethyst Cluster Fall', 'Amethyst Cluster Hit', 'Amethyst Cluster Place', 'Amethyst Cluster Step', 'Large Amethyst Bud Break', 'Large Amethyst Bud Place', 'Medium Amethyst Bud Break', 'Medium Amethyst Bud Place', 'Small Amethyst Bud Break', 'Small Amethyst Bud Place', 'Amethyst Resonate', 'Azalea Place', 'Azalea Step', 'Azalea Break', 'Azalea Fall', 'Azalea Hit', 'Azalea Leaves Break', 'Azalea Leaves Fall', 'Azalea Leaves Hit', 'Azalea Leaves Place', 'Azalea Leaves Step', 'Big Dripleaf Break', 'Big Dripleaf Fall', 'Big Dripleaf Hit', 'Big Dripleaf Break', 'Big Dripleaf Step', 'Big Dripleaf Tilt Down', 'Big Dripleaf Tilt Up', 'Calcite Break', 'Calcite Fall', 'Calcite Hit', 'Calcite Place', 'Calcite Step', 'Cave Vines Break', 'Cave Vines Fall', 'Cave Vines Hit', 'Cave Vines Pick', 'Cave Vines Place', 'Cave Vines Step', 'Deepslate Break', 'Deepslate Fall', 'Deepslate Hit', 'Deepslate Place', 'Deepslate Step', 'Deepslate Bricks Break', 'Deepslate Bricks Fall', 'Deepslate Bricks Hit', 'Deepslate Bricks Place', 'Deepslate Bricks Step', 'Deepslate Tiles Break', 'Deepslate Tiles Fall', 'Deepslate Tiles Hit', 'Deepslate Tiles Place', 'Deepslate Tiles Step', 'Dripstone Block Break', 'Dripstone Block Fall', 'Dripstone Block Hit', 'Dripstone Block Place', 'Dripstone Block Step', 'Flowering Azalea Break', 'Flowering Azalea Fall', 'Flowering Azalea Hit', 'Flowering Azalea Place', 'Flowering Azalea Step', 'Hanging Roots Break', 'Hanging Roots Fall', 'Hanging Roots Hit', 'Hanging Roots Place', 'Hanging Roots Step', 'Moss Block Break', 'Moss Block Fall', 'Moss Block Hit', 'Moss Block Place', 'Moss Block Step', 'Moss Carpet Break', 'Moss Carpet Fall', 'Moss Carpet Hit', 'Moss Carpet Place', 'Moss Carpet Step', 'Pointed Dripstone Break', 'Pointed Dripstone Drip Lava', 'Pointed Dripstone Drip Lava Into Cauldron', 'Pointed Dripstone Drip Water', 'Pointed Dripstone Drip Water Into Cauldron', 'Pointed Dripstone Fall', 'Pointed Dripstone Hit', 'Pointed Dripstone Land', 'Pointed Dripstone Place', 'Pointed Dripstone Step', 'Small Dripleaf Break', 'Small Dripleaf Fall', 'Small Dripleaf Hit', 'Small Dripleaf Place', 'Small Dripleaf Step', 'Sponge Absorb', 'Sponge Break', 'Sponge Fall', 'Sponge Hit', 'Sponge Place', 'Sponge Step', 'Wet Sponge Break', 'Wet Sponge Dries', 'Wet Sponge Fall', 'Wet Sponge Hit', 'Wet Sponge Place', 'Wet Sponge Step', 'Spore Blossom Break', 'Spore Blossom Fall', 'Spore Blossom Hit', 'Spore Blossom Place', 'Spore Blossom Step', 'Tuff Break', 'Tuff Fall', 'Tuff Hit', 'Tuff Place', 'Tuff Step', 'Sculk Break', 'Sculk Fall', 'Sculk Hit', 'Sculk Place', 'Sculk Step', 'Sculk Charge', 'Sculk Spread', 'Sculk Catalyst Break', 'Sculk Catalyst Fall', 'Sculk Catalyst Hit', 'Sculk Catalyst Place', 'Sculk Catalyst Step', 'Sculk Catalyst Bloom', 'Sculk Shrieker Break', 'Sculk Shrieker Fall', 'Sculk Shrieker Hit', 'Sculk Shrieker Place', 'Sculk Shrieker Step', 'Sculk Shrieker Shriek', 'Sculk Vein Break', 'Sculk Vein Fall', 'Sculk Vein Hit', 'Sculk Vein Place', 'Sculk Vein Step', 'Ancient Debris Break', 'Ancient Debris Fall', 'Ancient Debris Hit', 'Ancient Debris Place', 'Ancient Debris Step', 'Basalt Break', 'Basalt Fall', 'Basalt Hit', 'Basalt Place', 'Basalt Step', 'Gilded Blackstone Break', 'Gilded Blackstone Fall', 'Gilded Blackstone Hit', 'Gilded Blackstone Place', 'Gilded Blackstone Step', 'Netherrack Break', 'Netherrack Fall', 'Netherrack Hit', 'Netherrack Place', 'Netherrack Step', 'Nether Gold Ore Break', 'Nether Gold Ore Fall', 'Nether Gold Ore Hit', 'Nether Gold Ore Place', 'Nether Gold Ore Step', 'Nether Fungus Break', 'Nether Fungus Fall', 'Nether Fungus Hit', 'Nether Fungus Place', 'Nether Fungus Step', 'Nether Quartz Ore Break', 'Nether Quartz Ore Fall', 'Nether Quartz Ore Hit', 'Nether Quartz Ore Place', 'Nether Quartz Ore Step', 'Nether Roots Break', 'Nether Roots Fall', 'Nether Roots Hit', 'Nether Roots Place', 'Nether Roots Step', 'Nether Sprouts Break', 'Nether Sprouts Fall', 'Nether Sprouts Hit', 'Nether Sprouts Place', 'Nether Sprouts Step', 'Nether Stem Break', 'Nether Stem Fall', 'Nether Stem Hit', 'Nether Stem Place', 'Nether Stem Step', 'Nether Vine Break', 'Nether Vine Fall', 'Nether Vine Hit', 'Nether Vine Place', 'Nether Vine Step', 'Nether Wart Break', 'Nether Wart Plant', 'Nether Wart Block Break', 'Nether Wart Block Fall', 'Nether Wart Block Hit', 'Nether Wart Block Place', 'Nether Wart Block Step', 'Nylium Break', 'Nylium Fall', 'Nylium Hit', 'Nylium Place', 'Nylium Step', 'Shroomlight Break', 'Shroomlight Fall', 'Shroomlight Hit', 'Shroomlight Place', 'Shroomlight Step', 'Soul Sand Break', 'Soul Sand Fall', 'Soul Sand Hit', 'Soul Sand Place', 'Soul Sand Step', 'Soul Soil Break', 'Soul Soil Fall', 'Soul Soil Hit', 'Soul Soil Place', 'Soul Soil Step', 'Chorus Flower Death', 'Chorus Flower Grow', 'Add Cake Candle', 'Candle Ambient', 'Candle Break', 'Candle Extinguish', 'Candle Fall', 'Candle Hit', 'Candle Place', 'Candle Step', 'Chain Break', 'Chain Fall', 'Chain Hit', 'Chain Place', 'Chain Step', 'Copper Block Break', 'Copper Block Fall', 'Copper Block Hit', 'Copper Block Place', 'Copper Block Step', 'Decorated Pot Break', 'Decorated Pot Fall', 'Decorated Pot Hit', 'Decorated Pot Place', 'Decorated Pot Step', 'Decorated Pot Shatter', 'Decorated Pot Insert Item', 'Decorated Pot Insert Fail', 'Glass Break', 'Glass Fall', 'Glass Hit', 'Glass Place', 'Glass Step', 'Waxed Sign Interact Fail', 'Waxed Hanging Sign Interact Fail', 'Wooden Hanging Sign Break', 'Wooden Hanging Sign Fall', 'Wooden Hanging Sign Hit', 'Wooden Hanging Sign Place', 'Wooden Hanging Sign Step', 'Cherry Hanging Sign Break', 'Cherry Hanging Sign Fall', 'Cherry Hanging Sign Hit', 'Cherry Hanging Sign Place', 'Cherry Hanging Sign Step', 'Bamboo Hanging Sign Break', 'Bamboo Hanging Sign Fall', 'Bamboo Hanging Sign Hit', 'Bamboo Hanging Sign Place', 'Bamboo Hanging Sign Step', 'Nether Wood Hanging Sign Break', 'Nether Wood Hanging Sign Fall', 'Nether Wood Hanging Sign Hit', 'Nether Wood Hanging Sign Place', 'Nether Wood Hanging Sign Step', 'Heavy Core Break', 'Heavy Core Fall', 'Heavy Core Hit', 'Heavy Core Place', 'Heavy Core Step', 'Honey Block Break', 'Honey Block Fall', 'Honey Block Hit', 'Honey Block Place', 'Honey Block Slide', 'Honey Block Step', 'Ladder Break', 'Ladder Fall', 'Ladder Hit', 'Ladder Place', 'Ladder Step', 'Lantern Break', 'Lantern Fall', 'Lantern Hit', 'Lantern Place', 'Lantern Step', 'Metal Break', 'Metal Fall', 'Metal Hit', 'Metal Place', 'Metal Step', 'Nether Bricks Break', 'Nether Bricks Fall', 'Nether Bricks Hit', 'Nether Bricks Place', 'Nether Bricks Step', 'Netherite Block Break', 'Netherite Block Fall', 'Netherite Block Hit', 'Netherite Block Place', 'Netherite Block Step', 'Polished Deepslate Break', 'Polished Deepslate Fall', 'Polished Deepslate Hit', 'Polished Deepslate Place', 'Polished Deepslate Step', 'Polished Tuff Break', 'Polished Tuff Fall', 'Polished Tuff Hit', 'Polished Tuff Place', 'Polished Tuff Step', 'Scaffolding Break', 'Scaffolding Fall', 'Scaffolding Hit', 'Scaffolding Place', 'Scaffolding Step', 'Slime Block Break', 'Slime Block Fall', 'Slime Block Hit', 'Slime Block Place', 'Slime Block Step', 'Trial Spawner About to Spawn Item', 'Trial Spawner Ambient', 'Trial Spawner Ambient (Ominous)', 'Trial Spawner Break', 'Trial Spawner Close Shutter', 'Trial Spawner Detect Player', 'Trial Spawner Eject Item', 'Trial Spawner Fall', 'Trial Spawner Hit', 'Trial Spawner Ominous Activate', 'Trial Spawner Open Shutter', 'Trial Spawner Place', 'Trial Spawner Spawn Item', 'Trial Spawner Spawn Item Begin', 'Trial Spawner Spawn Mob', 'Trial Spawner Step', 'Tuff Bricks Break', 'Tuff Bricks Fall', 'Tuff Bricks Hit', 'Tuff Bricks Place', 'Tuff Bricks Step', 'Vault Activate', 'Vault Ambient', 'Vault Break', 'Vault Close Shutter', 'Vault Deactivate', 'Vault Eject Item', 'Vault Fall', 'Vault Hit', 'Vault Insert Item', 'Vault Insert Item (Fail)', 'Vault Open Shutter', 'Vault Place', 'Vault Reject Rewarded Player', 'Vault Step', 'Wool Break', 'Wool Fall', 'Wool Hit', 'Wool Place', 'Wool Step', 'Mud Bricks Break', 'Mud Bricks Fall', 'Mud Bricks Hit', 'Mud Bricks Place', 'Mud Bricks Step', 'Packed Mud Break', 'Packed Mud Fall', 'Packed Mud Hit', 'Packed Mud Place', 'Packed Mud Step', 'Anvil Break', 'Anvil Destroy', 'Anvil Fall', 'Anvil Hit', 'Anvil Land', 'Anvil Place', 'Anvil Step', 'Anvil Use', 'Blast Furnace Fire Crackle', 'Brewing Stand Brew', 'Campfire Crackle', 'Cartography Table Take Result', 'Composter Empty', 'Composter Fill Success', 'Composter Fill', 'Composter Ready', 'Enchantment Table Use', 'Furnace Fire Crackle', 'Grindstone Use', 'Loom Select Pattern', 'Loom Take Result', 'Smithing Table Use', 'Smoker Smoke', 'Stonecutter Select Recipe', 'Stonecutter Take Result', 'Barrel Close', 'Barrel Open', 'Chest Close', 'Chest Locked', 'Chest Open', 'Chiseled Bookshelf Break', 'Chiseled Bookshelf Fall', 'Chiseled Bookshelf Hit', 'Chiseled Bookshelf Place', 'Chiseled Bookshelf Step', 'Chiseled Bookshelf Insert', 'Chiseled Bookshelf Insert (Enchanted)', 'Chiseled Bookshelf Pick Up', 'Chiseled Bookshelf Pick Up (Enchanted)', 'Ender Chest Close', 'Ender Chest Open', 'Shulker Box Close', 'Shulker Box Open', 'Comparator Click', 'Dispenser Dispense', 'Dispenser Fail', 'Dispenser Launch', 'Wooden Fence Gate Close', 'Wooden Fence Gate Open', 'Cherry Fence Gate Close', 'Cherry Fence Gate Open', 'Bamboo Fence Gate Close', 'Bamboo Fence Gate Open', 'Nether Wood Fence Gate Close', 'Nether Wood Fence Gate Open', 'Iron Door Close', 'Iron Door Open', 'Iron Trapdoor Close', 'Iron Trapdoor Open', 'Lever Click', 'Metal Pressure Plate Click Off', 'Metal Pressure Plate Click On', 'Piston Retract', 'Piston Extend', 'Redstone Torch Burnout', 'Sculk Sensor Break', 'Sculk Sensor Click', 'Sculk Sensor Click Stop', 'Sculk Sensor Fall', 'Sculk Sensor Hit', 'Sculk Sensor Place', 'Sculk Sensor Step', 'Stone Button Click Off', 'Stone Button Click On', 'Stone Pressure Plate Click Off', 'Stone Pressure Plate Click On', 'Tripwire Attach', 'Tripwire Click Off', 'Tripwire Click On', 'Tripwire Detach', 'Wooden Button Click Off', 'Wooden Button Click On', 'Wooden Door Close', 'Wooden Door Open', 'Wooden Pressure Plate Click Off', 'Wooden Pressure Plate Click On', 'Wooden Trapdoor Close', 'Wooden Trapdoor Open', 'Cherry Wood Button Click Off', 'Cherry Wood Button Click On', 'Cherry Wood Door Close', 'Cherry Wood Door Open', 'Cherry Wood Pressure Plate Click Off', 'Cherry Wood Pressure Plate Click On', 'Cherry Wood Trapdoor Close', 'Cherry Wood Trapdoor Open', 'Bamboo Button Click Off', 'Bamboo Button Click On', 'Bamboo Door Close', 'Bamboo Door Open', 'Bamboo Pressure Plate Click Off', 'Bamboo Pressure Plate Click On', 'Bamboo Trapdoor Close', 'Bamboo Trapdoor Open', 'Nether Wood Button Click Off', 'Nether Wood Button Click On', 'Nether Wood Door Close', 'Nether Wood Door Open', 'Nether Wood Pressure Plate Click Off', 'Nether Wood Pressure Plate Click On', 'Nether Wood Trapdoor Close', 'Nether Wood Trapdoor Open', 'Bass Drum', 'Banjo', 'Bass', 'Bell', 'Bit', 'Chime', 'Cow Bell', 'Didgeridoo', 'Flute', 'Guitar', 'Harp', 'Pling', 'Hat', 'Snare Drum', 'Iron Xylophone', 'Xylophone', 'Creeper (Note Block)', 'Ender Dragon (Note Block)', 'Piglin (Note Block)', 'Skeleton (Note Block)', 'Wither Skeleton (Note Block)', 'Zombie (Note Block)', 'Beacon Activate', 'Beacon Ambient', 'Beacon Deactivate', 'Beacon Power Select', 'Bell Resonate', 'Bell Use', 'Conduit Activate', 'Conduit Ambient (Short)', 'Conduit Ambient', 'Conduit Attack Target', 'Conduit Deactivate', 'End Gateway Spawn', 'End Portal Fill Frame', 'End Portal Spawn', 'Lodestone Break', 'Lodestone Fall', 'Lodestone Hit', 'Lodestone Lock Compass', 'Lodestone Place', 'Lodestone Step', 'Nether Portal Ambient', 'Nether Portal Travel', 'Nether Portal Trigger', 'Respawn Anchor Ambient', 'Respawn Anchor Charge', 'Respawn Anchor Deplete', 'Respawn Anchor Set Spawnpoint', 'Cave Ambience', 'Lightning Impact', 'Lightning Thunder', 'Raid Horn', 'Underwater Ambience', 'Underwater Ambience Additions', 'Underwater Ambience Additions (Rare)', 'Underwater Ambience Additions (Very Rare)', 'Weather Rain', 'Weather Rain Above', 'Basalt Deltas Ambience', 'Basalt Deltas Ambience Additions', 'Basalt Deltas Ambience (Mood)', 'Crimson Forest Ambience', 'Crimson Forest Ambience Additions', 'Crimson Forest Ambience (Mood)', 'Nether Wastes Ambience', 'Nether Wastes Ambience Additions', 'Nether Wastes Ambience (Mood)', 'Soul Escape', 'Soul Sand Valley Ambience', 'Soul Sand Valley Ambience Additions', 'Soul Sand Valley Ambience (Mood)', 'Warped Forest Ambience', 'Warped Forest Ambience Additions', 'Warped Forest Ambience (Mood)']
POTION_NAME = Literal['Absorption', 'Conduit Power', "Dolphin's Grace", 'Fire Resistance', 'Haste', 'Health Boost', 'Hero of the Village', 'Instant Health', 'Invisibility', 'Jump Boost', 'Luck', 'Night Vision', 'Regeneration', 'Resistance', 'Saturation', 'Slow Falling', 'Strength', 'Speed', 'Water Breathing', 'Bad Omen', 'Blindness', 'Darkness', 'Glowing', 'Hunger', 'Infested', 'Instant Damage', 'Levitation', 'Mining Fatigue', 'Nausea', 'Oozing', 'Poison', 'Raid Omen', 'Slowness', 'Trial Omen', 'Bad Luck', 'Weakness', 'Weaving', 'Wind Charged', 'Wither']
REPEAT_SUBACTION = IF_PLAYER_ACTION | IF_ENTITY_ACTION | IF_GAME_ACTION | IF_VAR_ACTION
